#!/usr/bin/python

from brainart.utils import get_packagedir
base = get_packagedir()

def get_template(template_name,ext="html"):
    '''get_template
    :param template_name: name of template (without suffix) to get in the template package folder
    :param ext: extension of template (default is html)
    '''
    template_file = "%s/templates/%s.%s" %(base,template_name,ext)
    filey = open(template_file,"r")
    template = "\n".join(filey.readlines())
    filey.close()
    return template


def sub_template(template,template_tag,substitution):
    '''sub_template
    make a substitution for a template_tag in a template
    :param template: the template, read in as a str
    :param template_tag: the tag in the template, surrounded with {{}}
    :param substitution: the text (str) to substitute with
    '''
    template = template.replace("{{%s}}" %template_tag,substitution)
    return template

def save_template(output_file,html_snippet):
    '''save_template
    :param output_file: the output file to save the template to
    :param html_snippet: the template text to save
    '''
    filey = open(output_file,"w")
    filey.writelines(html_snippet)
    filey.close()



#!/usr/bin/python

from nilearn.plotting import plot_stat_map
from brainart.utils import get_packagedir
from numpy.random import choice
import matplotlib.pyplot as plt
from pyneurovault import api
import numpy as np
from glob import glob
import pandas
import numpy as np
import nibabel
import os

base = get_packagedir()

def get_lookup(lookup_name):
    lookups = glob("%s/data/*.pkl" %base)
    lookup_names = [os.path.basename(x).split(".")[0].split("_")[0] for x in lookups]
    if lookup_name not in lookup_names:
        print "Invalid lookup name, choices are %s." %(",".join(lookup_names))
    else:
        return pandas.read_pickle("%s/data/%s_lookup.pkl" %(base,lookup_name))

def save_lookup(png_images,output_pkl):
    '''save_lookup
    save a color lookup table based on a set of images
    :param png_images: a list of full paths to png images to save
    :param output_pkl: the color lookup table will be saved to this pickle file
    '''
    lookup_table = get_color_lookup(png_images)
    lookup_table.to_pickle(output_pkl)


def get_data(download_folder):
    all_collections = api.get_collections()
    collections_with_dois = all_collections[np.logical_not(all_collections.DOI.isnull())]
    group_collections = collections_with_dois[collections_with_dois.group_comparison!=True]
    images = api.get_images(collection_pks=list(group_collections.collection_id))
    images = api.filter(df=images,column_name="is_thresholded",field_value=False)
    images = api.filter(df=images,column_name="not_mni",field_value=False)
    images = api.filter(df=images,column_name="modality",field_value="fMRI-BOLD")
    api.download_images(dest_dir = download_folder,images_df=images,resample=False)
    brainmaps = glob("%s/original/*.nii.gz" %(download_folder))
    output_folder = "%s/png" %(download_folder)
    if not os.path.exists(output_folder):
        os.mkdir(output_folder)

    # Let's use a crapton of colormaps
    colormaps = ["Spectral", "summer", "coolwarm", "Wistia_r", "pink_r", "Set1", "Set2", "Set3", "brg_r", "Dark2", "prism", "PuOr_r", "afmhot_r", "terrain_r", "PuBuGn_r", "RdPu", "gist_ncar_r", "gist_yarg_r", "Dark2_r", "YlGnBu", "RdYlBu", "hot_r", "gist_rainbow_r", "gist_stern", "PuBu_r", "cool_r", "cool", "gray", "copper_r", "Greens_r", "GnBu", "gist_ncar", "spring_r", "gist_rainbow", "gist_heat_r", "Wistia", "OrRd_r", "CMRmap", "bone", "gist_stern_r", "RdYlGn", "Pastel2_r", "spring", "terrain", "YlOrRd_r", "Set2_r", "winter_r", "PuBu", "RdGy_r", "spectral", "rainbow", "flag_r", "jet_r", "RdPu_r", "gist_yarg", "BuGn", "Paired_r", "hsv_r", "bwr", "cubehelix", "Greens", "PRGn", "gist_heat", "spectral_r", "Paired", "hsv", "Oranges_r", "prism_r", "Pastel2", "Pastel1_r", "Pastel1", "gray_r", "jet", "Spectral_r", "gnuplot2_r", "gist_earth", "YlGnBu_r", "copper", "gist_earth_r", "Set3_r", "OrRd", "gnuplot_r", "ocean_r", "brg", "gnuplot2", "PuRd_r", "bone_r", "BuPu", "Oranges", "RdYlGn_r", "PiYG", "CMRmap_r", "YlGn", "binary_r", "gist_gray_r", "Accent", "BuPu_r", "gist_gray", "flag", "bwr_r", "RdBu_r", "BrBG", "Reds", "Set1_r", "summer_r", "GnBu_r", "BrBG_r", "Reds_r", "RdGy", "PuRd", "Accent_r", "Blues", "autumn_r", "autumn", "cubehelix_r", "nipy_spectral_r", "ocean", "PRGn_r", "Greys_r", "pink", "binary", "winter", "gnuplot", "RdYlBu_r", "hot", "YlOrBr", "coolwarm_r", "rainbow_r", "Purples_r", "PiYG_r", "YlGn_r", "Blues_r", "YlOrBr_r", "seismic", "Purples", "seismic_r", "RdBu", "Greys", "BuGn_r", "YlOrRd", "PuOr", "PuBuGn", "nipy_spectral", "afmhot"]

    # We want an axial slice with different renderings (all the colors!)
    colormap_choices = colormaps[:]
    for brainmap in brainmaps:
        image_id = os.path.basename(brainmap).replace(".nii.gz","")
        mr = nibabel.load(brainmap)
        if len(colormap_choices)==0:
            colormap_choices = colormaps[:]
        color = colormap_choices.pop()
        # Save a black and a white version
        plot_stat_map(mr,display_mode="z",colorbar=False,annotate=False,draw_cross=False,cmap=color,cut_coords=1)
        plt.savefig("%s/%s_white.png" %(output_folder,image_id))
        plt.close()
        plot_stat_map(mr,display_mode="z",colorbar=False,annotate=False,draw_cross=False,cmap=color,cut_coords=1,black_bg=True)
        plt.savefig("%s/%s_black.png" %(output_folder,image_id),facecolor="k", edgecolor="k")
        plt.close()




#!/usr/bin/python

from brainart.utils import get_avg_rgb, get_color_lookup, generate_matching_df, get_packagedir
from brainart.template import get_template, sub_template, save_template
from glob import glob
import numpy as np
import webbrowser
import tempfile
import pandas
import json
import sys
import os

base = get_packagedir() 

def generate(template,output_folder=None,color_lookup="white",image_base_path=None,bgcolor="black",threshold=0.9,sample=5):
    '''generate
    create a brainart image using a particular color lookup table
    :param template: the template image to generate brainart for! Only jpg has been tested.
    :param output_folder: output folder for the final file. If not specified, will use TMP
    :param color_lookup: one of "white" "black" "greywhite" or "greyblack" will default to using brainart database, and you do not need to worry about setting image_base_path. If you set this to a custom lookup table (should be a pandas dataframe) then you also need to set image_base_path to be the directory with images defined in your lookup table index. Default color lookup is white.
    :param image_base_path: Only specify if you provide a custom color lookup pandas. Default is None, as the base path is the github repo images. 
    :param bgcolor: background color (string or hex) for the output html page. Default is black, and will be matched to color lookup if one is provided.
    :param threshold: The threshold to use when assessing similarity of a pixel to a mean brain image color. Default is 0.9
    :param sample: how many pixels to sample from. Default is every 5th pixel (5). Larger numbers mean generating images faster.
    '''
    if output_folder == None:
        output_folder = tempfile.mkdtemp()

    if isinstance(color_lookup,pandas.DataFrame):
        print "Custom color lookup found!"
        if image_base_path == None:
            print "Error: color lookup is set to custom file but image_base_path is set to 'None.' If you want to use images for a custom table you have created, you need to provide a base path to those images."
            sys.exit()
    elif color_lookup not in ["white","black","greywhite","greyblack"]:
        print "Error: package color lookup tables include 'white','black','greywhite' and 'greyblack'"
        sys.exit()
    else:
        # Background color of html page should be black in all cases except for white lookup
        if color_lookup == "white":
            bgcolor = "white"        
        color_lookup = pandas.read_pickle("%s/data/%s_lookup.pkl" %(base,color_lookup))
        image_base_path = "https://rawgithub.com/vsoch/brainart/master/png"

    print "Generating image... this can take a few minutes."
    new_image = generate_matching_df(template,color_lookup,threshold=threshold,sample=sample)

    # Make paths relative to image_base_path
    new_image.png = ["%s/%s" %(image_base_path,x) for x in new_image.png]

    # Get image template
    html_template = get_template("index")
    image_name = os.path.splitext(os.path.basename(template))[0]

    # Save output to file
    new_image_dict = new_image.to_dict(orient="records")
    result = {"image":new_image_dict,"bgcolor":bgcolor}
    html_template = sub_template(html_template,"DATA",str(result))
    output_html = "%s/index.html" %output_folder
    save_template(output_html,html_template)
        
    print "Result files being saved to %s" %output_folder
    webbrowser.open_new_tab(output_html)




#!/usr/bin/python

import numpy as np
from numpy.random import choice
from PIL import Image
import pandas
import os
import brainart.hello as hello

def get_packagedir():
    return os.path.dirname(hello.__file__)

def get_avg_rgb(png_image):
    """
    get_avg_rgb:
        returns a tuple with (True/False,R,G,B)
        True/False indicates if the image is not empty
        the R G B are red, green, blue values, respectively
    """

    img = Image.open(png_image)
    width, height = img.size

    # make a list of all pixels in the image
    pixels = img.load()
    data = []
    for x in range(width):
        for y in range(height):
            cpixel = pixels[x, y]
            data.append(cpixel)

    r = 0
    g = 0
    b = 0
    counter = 0

    # loop through all pixels
    # if alpha value is greater than 200/255, add it to the average
    for x in range(len(data)):
        if (data[x][3] > 200):
            if not (data[x][0] == 0 and data[x][1] == 0 and data[x][2] == 0):
                # Don't count white, black is 0 so we don't care
                if not (data[x][0] == 255 and data[x][1] == 255 and data[x][2] == 255):
                    r+=data[x][0]
                    g+=data[x][1]
                    b+=data[x][2]
                    counter+=1;

    # compute average RGB values
    if counter != 0:
        rAvg = r/counter
        gAvg = g/counter
        bAvg = b/counter
        return (True,rAvg, gAvg, bAvg)
    else:
        return (False,0,0,0)


def get_color_lookup(png_images,remove_path=True):
    '''get_color_lookup
    returns pandas data frame with average color of an image. Columns are "R" "G" "B" and rows are image paths
    :param png_images: full paths to png images to include
    :param remove_path: if True, will return table with image names only (default is True)
    '''
    color_lookup = pandas.DataFrame(columns=["R","G","B"])
    for png_image in png_images:
        valid,R,G,B = get_avg_rgb(png_image)
        if valid:
            if remove_path == True:
                png_image = os.path.basename(png_image)
            color_lookup.loc[png_image] = [R,G,B]
    return color_lookup


def generate_matching_df(template,color_lookup,threshold=0.9,sample=10):
    """
     generate_matching_df (worst function name ever)
        this will generate a dataframe with x,y, corr, and png file path
        for images that most highly  
    """
    # Now read in our actual image
    base = Image.open(template)
    width, height = base.size
    pixels = base.load()
    data = []

    count=0
    new_image = pandas.DataFrame(columns=["x","y","corr","png"])
    for x in range(width):
        for y in range(height):
            # And take only every [sample]th pixel
            if np.remainder(x,sample)==0 and np.remainder(y,sample)==0:
                cpixel = pixels[x, y]
                try:
                    # Make a temporary data frame with cpixel appended
                    tmp = color_lookup.copy()
                    tmp.loc["lookup"] = cpixel[0:3]
                    # Calculate pairwise correlations
                    corrs = tmp.transpose().corr()
                    scores = corrs.loc["lookup"]
                    scores = scores.drop(["lookup"])
                    scores = scores[scores.isnull()==False]
                    scores.sort(ascending=False)
                    # Randomly sample from anything > 0.9
                    png = choice(scores[scores>threshold].index.tolist(),1)[0]
                    new_image.loc[count] = [x,y,scores[png],png]
                except:
                    png = choice(color_lookup.index,1)[0]
                    new_image.loc[count] = [x,y,0.0,png] 
                count+=1
    return new_image




#!/usr/bin/python

'''
script.py: 
Runtime executable

'''

from brainart.utils import get_avg_rgb, get_color_lookup, generate_matching_df
from brainart.db import get_data
from brainart.mosaic import generate
from glob import glob
import numpy as np
import argparse
import pandas
import sys
import os


def main():
    parser = argparse.ArgumentParser(
    description="make images out of brain imaging data")
    parser.add_argument("--input", dest='image', help="full path to jpg image", type=str, default=None,required=True)
    parser.add_argument("--db", dest='db', help="path to folder for png images for database", type=str, default=None)
    parser.add_argument("--sample", dest='sample', help="sample every SAMPLEth pixel", type=int, default=10)
    parser.add_argument("--threshold", dest='threshold', help="threshold value to match pixels to brain images", type=float, default=0.9)
    parser.add_argument('--update', dest='update', help="regenerate png database", default=False, action='store_true')
    parser.add_argument("--background-color", dest='bgcolor', help="background color", type=str, default="black")
    parser.add_argument("--color-lookup", dest='lookup', help="color lookup (white, black) which currently determined background color.", type=str, default="white")
    parser.add_argument("--output-folder", dest='output', help="output folder for html file", type=str, default=None)

    try:
        args = parser.parse_args()
    except:
        parser.print_help()
        sys.exit(0)
    
    # Regenerate database of png images
    if args.update == True:
        if args.db is not None:
            get_data(args.folder)
        else:
            print "Please specify location for png images for database with --db argument"
    else:        
        if args.image == None:
            print "Please specify input jpg image with --input argument."

        # Check background color
        if args.lookup not in ["black","white"]:
            print "Unrecognized lookup table! Setting to white."
            args.lookup = "white"
            
        generate(template=args.image,
                 output_folder=args.output,
                 color_lookup=args.lookup,
                 bgcolor=args.bgcolor,
                 threshold=args.threshold,
                 sample=args.sample)



from pyneurovault import api
import numpy as np

all_collections = api.get_collections()
collections_with_dois = all_collections[np.logical_not(all_collections.DOI.isnull())]

images = api.get_images(collection_pks=list(collections_with_dois.collection_id))

# Remove images that are thresholded
images = api.filter(df=images,column_name="is_thresholded",field_value=False)

# Not in MNI
images = api.filter(df=images,column_name="not_mni",field_value=False)

# Just fMRI bold
images = api.filter(df=images,column_name="modality",field_value="fMRI-BOLD")

# Download all images to file, resample to target
outfolder = "/home/vanessa/Documents/Work/NEUROVAULT/mosaic"

# If you don't want to resample
api.download_images(dest_dir = outfolder,images_df=images,resample=False)



from setuptools import setup, find_packages
import codecs
import os

setup(
    # Application name:
    name="brainart",

    # Version number (initial):
    version="1.0.2",

    # Application author details:
    author="vsoch",
    author_email="vsochat@stanford.edu",

    # Packages
    packages=find_packages(),

    # Data files
    package_data = {'brainart.templates':['*.html'],
                    'brainart.data':['*.pkl']},

    include_package_data=True,
    zip_safe=False,

    # Details
    url="http://www.github.com/vsoch/brainart",

    license="LICENSE.txt",
    description="make images out of brain imaging data",
    keywords='brain art neuroimaging',

    install_requires = ['numpy','pandas','pyneurovault','nilearn','matplotlib'],

    entry_points = {
        'console_scripts': [
            'brainart=brainart.scripts:main',
        ],
    },

)
